@@ -13,21 +13,27 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from http import client as http_client
 from io import BytesIO
-from oslo_config import cfg
-from oslo_log import log as logging
-from oslo_utils import excutils
-from oslo_utils import uuidutils
-import six
-from six.moves import http_client
+import json
+import mimetypes
+import os
 import webob
 import zipfile
 from zipfile import ZipFile
 
+from glance_store import exceptions as store_exceptions
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import encodeutils
+from oslo_utils import excutils
+from oslo_utils import uuidutils
+
 from tacker._i18n import _
 from tacker.api.schemas import vnf_packages
 from tacker.api import validation
 from tacker.api.views import vnf_packages as vnf_packages_view
+from tacker.common import csar_utils
 from tacker.common import exceptions
 from tacker.common import utils
 from tacker.conductor.conductorrpc import vnf_pkgm_rpc
@@ -84,7 +90,10 @@ class VnfPkgmController(wsgi.Controller):
 
         vnf_package.create()
 
-        return self._view_builder.create(request, vnf_package)
+        headers = {"location": '/vnfpkgm/v1/vnf_packages/%s' % vnf_package.id}
+        result = self._view_builder.create(vnf_package)
+
+        return wsgi.ResponseObject(result, headers=headers)
 
     @wsgi.response(http_client.OK)
     @wsgi.expected_errors((http_client.FORBIDDEN, http_client.NOT_FOUND))
@@ -99,13 +108,13 @@ class VnfPkgmController(wsgi.Controller):
 
         try:
             vnf_package = vnf_package_obj.VnfPackage.get_by_id(
-                request.context, id,
-                expected_attrs=["vnf_deployment_flavours", "vnfd"])
+                request.context, id, expected_attrs=[
+                    "vnf_deployment_flavours", "vnfd", "vnf_artifacts"])
         except exceptions.VnfPackageNotFound:
             msg = _("Can not find requested vnf package: %s") % id
             raise webob.exc.HTTPNotFound(explanation=msg)
 
-        return self._view_builder.show(request, vnf_package)
+        return self._view_builder.show(vnf_package)
 
     @wsgi.response(http_client.OK)
     @wsgi.expected_errors((http_client.BAD_REQUEST, http_client.FORBIDDEN))
@@ -142,9 +151,11 @@ class VnfPkgmController(wsgi.Controller):
         vnf_packages = vnf_package_obj.VnfPackagesList.get_by_filters(
             request.context, read_deleted='no', filters=filters)
 
-        return self._view_builder.index(request, vnf_packages,
-                all_fields=all_fields, exclude_fields=exclude_fields,
-                fields=fields, exclude_default=exclude_default)
+        return self._view_builder.index(vnf_packages,
+                                        all_fields=all_fields,
+                                        exclude_fields=exclude_fields,
+                                        fields=fields,
+                                        exclude_default=exclude_default)
 
     @wsgi.response(http_client.NO_CONTENT)
     @wsgi.expected_errors((http_client.FORBIDDEN, http_client.NOT_FOUND,
@@ -249,7 +260,7 @@ class VnfPkgmController(wsgi.Controller):
                                                     zip_file_size)
         else:
             chunk_size = zip_file_size
-        response.headers['Content-Length'] = six.text_type(chunk_size)
+        response.headers['Content-Length'] = str(chunk_size)
         return response
 
     def _get_csar_zip_data(self, uuid, location, offset=0, chunk_size=None):
@@ -274,8 +285,8 @@ class VnfPkgmController(wsgi.Controller):
             range_ = webob.byterange.Range.parse(range_str)
             if range_ is None:
                 range_err_msg = _("The byte range passed in the 'Range' header"
-                 "did not match any available byte range in the VNF package"
-                 "file")
+                 " did not match any available byte range in the VNF package"
+                 " file")
                 raise webob.exc.HTTPRequestRangeNotSatisfiable(
                     explanation=range_err_msg)
             # NOTE(sameert): Ensure that a range like bytes=4- for an zip
@@ -295,19 +306,37 @@ class VnfPkgmController(wsgi.Controller):
         context = request.environ['tacker.context']
         context.can(vnf_package_policies.VNFPKGM % 'upload_package_content')
 
-        vnf_package = self._get_vnf_package(id, request)
+        # check if id is of type uuid format
+        if not uuidutils.is_uuid_like(id):
+            msg = _("Can not find requested vnf package: %s") % id
+            return self._make_problem_detail('Not Found', msg, 404)
+
+        try:
+            vnf_package = vnf_package_obj.VnfPackage.get_by_id(
+                request.context, id)
+        except exceptions.VnfPackageNotFound:
+            msg = _("Can not find requested vnf package: %s") % id
+            return self._make_problem_detail('Not Found', msg, 404)
+        except Exception as e:
+            return self._make_problem_detail(
+                'Internal Server Error', str(e), 500)
 
         if vnf_package.onboarding_state != \
            fields.PackageOnboardingStateType.CREATED:
             msg = _("VNF Package %(id)s onboarding state "
                     "is not %(onboarding)s")
-            raise webob.exc.HTTPConflict(explanation=msg % {"id": id,
-                    "onboarding": fields.PackageOnboardingStateType.CREATED})
+            return self._make_problem_detail('Conflict', msg % {"id": id,
+                    "onboarding": fields.PackageOnboardingStateType.CREATED},
+                    409)
 
         vnf_package.onboarding_state = (
             fields.PackageOnboardingStateType.UPLOADING)
 
-        vnf_package.save()
+        try:
+            vnf_package.save()
+        except Exception as e:
+            return self._make_problem_detail(
+                'Internal Server Error', str(e), 500)
 
         try:
             (location, size, checksum, multihash,
@@ -316,16 +345,29 @@ class VnfPkgmController(wsgi.Controller):
             with excutils.save_and_reraise_exception():
                 vnf_package.onboarding_state = (
                     fields.PackageOnboardingStateType.CREATED)
-                vnf_package.save()
-
-        vnf_package.onboarding_state = (
-            fields.PackageOnboardingStateType.PROCESSING)
+                try:
+                    vnf_package.save()
+                except Exception as e:
+                    return self._make_problem_detail(
+                        'Internal Server Error', str(e), 500)
 
         vnf_package.algorithm = CONF.vnf_package.hashing_algorithm
         vnf_package.hash = multihash
         vnf_package.location_glance_store = location
         vnf_package.size = size
-        vnf_package.save()
+        try:
+            vnf_package.save()
+        except Exception as e:
+            vnf_package.onboarding_state = (
+                fields.PackageOnboardingStateType.CREATED)
+            try:
+                vnf_package.save()
+            except Exception as e:
+                return self._make_problem_detail(
+                    'Internal Server Error', str(e), 500)
+
+            return self._make_problem_detail(
+                'Internal Server Error', str(e), 500)
 
         # process vnf_package
         self.rpc_api.upload_vnf_package_content(context, vnf_package)
@@ -451,8 +493,7 @@ class VnfPkgmController(wsgi.Controller):
                 get_vnf_package_vnfd(context, vnf_package)
         except exceptions.FailedToGetVnfdData as e:
             LOG.error(e.msg)
-            raise webob.exc.HTTPInternalServerError(
-                explanation=six.text_type(e.msg))
+            raise webob.exc.HTTPInternalServerError(explanation=str(e.msg))
 
         if 'text/plain' in accept_headers:
             # Checking for yaml files only. This is required when there is
@@ -475,6 +516,134 @@ class VnfPkgmController(wsgi.Controller):
             request.response.headers['Content-Type'] = 'application/zip'
             return self._create_vnfd_zip(vnfd_files_and_data)
 
+    @wsgi.response(http_client.OK)
+    @wsgi.expected_errors((http_client.BAD_REQUEST, http_client.FORBIDDEN,
+                           http_client.NOT_FOUND, http_client.CONFLICT,
+                           http_client.REQUESTED_RANGE_NOT_SATISFIABLE))
+    def fetch_vnf_package_artifacts(self, request, id, artifact_path):
+        context = request.environ['tacker.context']
+        # get policy
+        context.can(vnf_package_policies.VNFPKGM % 'fetch_artifact')
+
+        # get vnf_package
+        if not uuidutils.is_uuid_like(id):
+            msg = _("Can not find requested vnf package: %s") % id
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        try:
+            vnf_package = vnf_package_obj.VnfPackage.get_by_id(
+                request.context, id,
+                expected_attrs=["vnf_artifacts"])
+        except exceptions.VnfPackageNotFound:
+            msg = _("Can not find requested vnf package: %s") % id
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+        if vnf_package.onboarding_state != \
+                fields.PackageOnboardingStateType.ONBOARDED:
+            msg = _("VNF Package %(id)s state is not "
+                    "%(onboarded)s")
+            raise webob.exc.HTTPConflict(explanation=msg % {"id": id,
+                    "onboarded": fields.PackageOnboardingStateType.ONBOARDED})
+
+        offset, chunk_size = 0, None
+
+        # get all artifact's path
+        artifact_file_paths = []
+        for item in vnf_package.vnf_artifacts:
+            artifact_file_paths.append(item.artifact_path)
+
+        if artifact_path in artifact_file_paths:
+            # get file's size
+            csar_path = self._get_csar_path(vnf_package)
+            absolute_artifact_path = os.path.join(csar_path, artifact_path)
+            if not os.path.isfile(absolute_artifact_path):
+                msg = _(
+                    "This type of path(url) '%s' is currently not supported") \
+                    % artifact_path
+                raise webob.exc.HTTPBadRequest(explanation=msg)
+            artifact_size = os.path.getsize(absolute_artifact_path)
+            range_val = self._get_range_from_request(request, artifact_size)
+            # range_val exists
+            if range_val:
+                if isinstance(range_val, webob.byterange.Range):
+                    # get the position of the last byte in the artifact file
+                    response_end = artifact_size - 1
+                    if range_val.start >= 0:
+                        offset = range_val.start
+                    else:
+                        if abs(range_val.start) < artifact_size:
+                            offset = artifact_size + range_val.start
+                    if range_val.end is not None and \
+                            range_val.end < artifact_size:
+                        chunk_size = range_val.end - offset
+                        response_end = range_val.end - 1
+                    else:
+                        chunk_size = artifact_size - offset
+                request.response.status_int = 206
+            # range_val does not exist, download the whole content of file
+            else:
+                offset = 0
+                chunk_size = artifact_size
+
+            # get file's mineType;
+            mime_type = mimetypes.guess_type(artifact_path.split('/')[-1])[0]
+            if mime_type:
+                request.response.headers['Content-Type'] = mime_type
+            else:
+                request.response.headers['Content-Type'] = \
+                    'application/octet-stream'
+            try:
+                artifact_data = self._download_vnf_artifact(
+                    absolute_artifact_path, offset, chunk_size)
+            except exceptions.FailedToGetVnfArtifact as e:
+                LOG.error(e.msg)
+                raise webob.exc.HTTPInternalServerError(
+                    explanation=e.msg)
+            request.response.text = artifact_data.decode('utf-8')
+            if request.response.status_int == 206:
+                request.response.headers['Content-Range'] = 'bytes %s-%s/%s' \
+                                                            % (offset,
+                                                               response_end,
+                                                               artifact_size)
+            else:
+                chunk_size = artifact_size
+
+            request.response.headers['Content-Length'] = chunk_size
+            return request.response
+        else:
+            msg = _("Not Found Artifact File.")
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
+    def _get_csar_path(self, vnf_package):
+        csar_path = os.path.join(CONF.vnf_package.vnf_package_csar_path,
+                                 vnf_package.id)
+
+        if not os.path.isdir(csar_path):
+            location = vnf_package.location_glance_store
+            try:
+                zip_path = glance_store.load_csar(vnf_package.id, location)
+                csar_utils.extract_csar_zip_file(zip_path, csar_path)
+            except (store_exceptions.GlanceStoreException) as e:
+                exc_msg = encodeutils.exception_to_unicode(e)
+                msg = (_("Exception raised from glance store can be "
+                         "unrecoverable if it is not related to connection"
+                         " error. Error: %s.") % exc_msg)
+                raise exceptions.FailedToGetVnfArtifact(error=msg)
+        return csar_path
+
+    def _download_vnf_artifact(self, artifact_file_path, offset=0,
+            chunk_size=None):
+        try:
+            with open(artifact_file_path, 'rb') as f:
+                f.seek(offset, 1)
+                vnf_artifact_data = f.read(chunk_size)
+                return vnf_artifact_data
+        except Exception as e:
+            exc_msg = encodeutils.exception_to_unicode(e)
+            msg = (_("Exception raised while reading artifact file"
+                     " Error: %s.") % exc_msg)
+            raise exceptions.FailedToGetVnfArtifact(error=msg)
+
     def _create_vnfd_zip(self, vnfd_files_and_data):
         buff = BytesIO()
         with ZipFile(buff, 'w', zipfile.ZIP_DEFLATED) as zip_archive:
@@ -483,6 +652,16 @@ class VnfPkgmController(wsgi.Controller):
 
         return buff.getvalue()
 
+    def _make_problem_detail(self, title, detail, status):
+        res = webob.Response(content_type='application/problem+json')
+        problemDetails = {}
+        problemDetails['title'] = title
+        problemDetails['detail'] = detail
+        problemDetails['status'] = status
+        res.text = json.dumps(problemDetails)
+        res.status_int = status
+        return res
+
 
 def create_resource():
     body_deserializers = {
