@@ -13,14 +13,19 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from datetime import datetime
 import weakref
 
-from six import iteritems
+from oslo_log import log as logging
+from sqlalchemy.orm import exc as orm_exc
 from sqlalchemy import sql
 
+from tacker._i18n import _
 from tacker.common import exceptions as n_exc
 from tacker.db import sqlalchemyutils
 
+LOG = logging.getLogger(__name__)
+
 
 class CommonDbMixin(object):
     """Common methods used in core and service plugins."""
@@ -83,16 +88,18 @@ class CommonDbMixin(object):
                                 (model.shared == sql.true()))
             else:
                 query_filter = (model.tenant_id == context.tenant_id)
+
         # Execute query hooks registered from mixins and plugins
-        for _name, hooks in iteritems(self._model_query_hooks.get(model, {})):
+        model_hooks = self._model_query_hooks.get(model, {})
+        for _name, hooks in model_hooks.items():
             query_hook = hooks.get('query')
-            if isinstance(query_hook, basestring):
+            if isinstance(query_hook, str):
                 query_hook = getattr(self, query_hook, None)
             if query_hook:
                 query = query_hook(context, model, query)
 
             filter_hook = hooks.get('filter')
-            if isinstance(filter_hook, basestring):
+            if isinstance(filter_hook, str):
                 filter_hook = getattr(self, filter_hook, None)
             if filter_hook:
                 query_filter = filter_hook(context, model, query_filter)
@@ -104,7 +111,7 @@ class CommonDbMixin(object):
 
         # Don't list the deleted entries
         if hasattr(model, 'deleted_at'):
-            query = query.filter_by(deleted_at=None)
+            query = query.filter_by(deleted_at=datetime.min)
 
         return query
 
@@ -131,14 +138,14 @@ class CommonDbMixin(object):
 
     def _apply_filters_to_query(self, query, model, filters):
         if filters:
-            for key, value in iteritems(filters):
+            for key, value in filters.items():
                 column = getattr(model, key, None)
                 if column:
-                    query = query.filter(column.in_(value))
-            for _name, hooks in iteritems(
-                    self._model_query_hooks.get(model, {})):
+                    query = query.filter(column.in_([value]))
+            model_hooks = self._model_query_hooks.get(model, {})
+            for _name, hooks in model_hooks.items():
                 result_filter = hooks.get('result_filters', None)
-                if isinstance(result_filter, basestring):
+                if isinstance(result_filter, str):
                     result_filter = getattr(self, result_filter, None)
 
                 if result_filter:
@@ -151,7 +158,7 @@ class CommonDbMixin(object):
         for func in self._dict_extend_functions.get(
                 resource_type, []):
             args = (response, db_object)
-            if isinstance(func, basestring):
+            if isinstance(func, str):
                 func = getattr(self, func, None)
             else:
                 # must call unbound method - use self as 1st argument
@@ -199,5 +206,15 @@ class CommonDbMixin(object):
         the model passed as second parameter.
         """
         columns = [c.name for c in model.__table__.columns]
-        return dict((k, v) for (k, v) in
-                    iteritems(data) if k in columns)
+        return dict((k, v) for (k, v) in data.items() if k in columns)
+
+    def _get_by_name(self, context, model, name):
+        try:
+            query = self._model_query(context, model)
+            return query.filter(model.name == name).one()
+        except orm_exc.NoResultFound:
+            LOG.info("No result found for %(name)s in %(model)s table",
+                     {'name': name, 'model': model})
+
+    def get_by_name(self, context, model, name):
+        return self._get_by_name(context, model, name)
