@@ -22,7 +22,6 @@ from oslo_utils import versionutils
 from oslo_versionedobjects import base as ovoo_base
 from sqlalchemy.orm import joinedload
 from sqlalchemy.sql import func
-from sqlalchemy_filters import apply_filters
 
 from tacker._i18n import _
 from tacker.common import exceptions
@@ -32,12 +31,17 @@ from tacker.db.db_sqlalchemy import api
 from tacker.db.db_sqlalchemy import models
 from tacker import objects
 from tacker.objects import base
+from tacker.objects import common
 from tacker.objects import fields
+from tacker.objects import vnf_artifact
 from tacker.objects import vnf_software_image
 
 _NO_DATA_SENTINEL = object()
 
-VNF_PACKAGE_OPTIONAL_ATTRS = ['vnf_deployment_flavours', 'vnfd']
+VNF_PACKAGE_OPTIONAL_ATTRS = [
+    'vnf_deployment_flavours',
+    'vnfd',
+    'vnf_artifacts']
 
 LOG = logging.getLogger(__name__)
 
@@ -178,7 +182,13 @@ def _vnf_package_list_by_filters(context, read_deleted=None, filters=None):
             query = query.join(models.VnfDeploymentFlavour).join(
                 models.VnfSoftwareImage)
 
-        query = apply_filters(query, filters)
+        if 'VnfPackageArtifactInfo' in filter_data:
+            query = query.join(models.VnfPackageArtifactInfo)
+
+        if 'VnfPackageVnfd' in filter_data:
+            query = query.join(models.VnfPackageVnfd)
+
+        query = common.apply_filters(query, filters)
 
     return query.all()
 
@@ -215,17 +225,21 @@ def _destroy_vnf_package(context, package_uuid):
         package_uuid=package_uuid)
 
     software_images_query = api.model_query(
-        context, models.VnfSoftwareImage,
-        (models.VnfSoftwareImage.id, )).filter(
-        models.VnfSoftwareImage.flavour_uuid.in_(flavour_query.subquery()))
+        context, models.VnfSoftwareImage, (models.VnfSoftwareImage.id, )
+    ).filter(models.VnfSoftwareImage.flavour_uuid.in_(
+        flavour_query.subquery().select()))
 
     api.model_query(
-        context, models.VnfSoftwareImageMetadata).filter(
-        models.VnfSoftwareImageMetadata.image_uuid.in_(
-            software_images_query.subquery())).update(
-        updated_values, synchronize_session=False)
+        context, models.VnfSoftwareImageMetadata
+    ).filter(models.VnfSoftwareImageMetadata.image_uuid.in_(
+        software_images_query.subquery().select())
+    ).update(updated_values, synchronize_session=False)
 
-    software_images_query.update(updated_values, synchronize_session=False)
+    api.model_query(
+        context, models.VnfSoftwareImage
+    ).filter(models.VnfSoftwareImage.id.in_(
+        flavour_query.subquery().select())
+    ).update(updated_values, synchronize_session=False)
 
     api.model_query(context, models.VnfPackageUserData). \
         filter_by(package_uuid=package_uuid). \
@@ -233,6 +247,9 @@ def _destroy_vnf_package(context, package_uuid):
     api.model_query(context, models.VnfDeploymentFlavour). \
         filter_by(package_uuid=package_uuid). \
         update(updated_values, synchronize_session=False)
+    api.model_query(context, models.VnfPackageArtifactInfo). \
+        filter_by(package_uuid=package_uuid). \
+        update(updated_values, synchronize_session=False)
     api.model_query(context, models.VnfPackageVnfd). \
         filter_by(package_uuid=package_uuid). \
         soft_delete(synchronize_session=False)
@@ -270,7 +287,7 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
     # 4. Valid values for a given data type if any. This value is set
     # especially for 'enum' data type.
     ALL_ATTRIBUTES = {
-        'id': ('id', "string", 'VnfPackage'),
+        'id': ('id', "uuid", 'VnfPackage'),
         'onboardingState': ('onboarding_state', "enum", 'VnfPackage',
             fields.PackageOnboardingStateTypeField().valid_values),
         'operationalState': ('operational_state', 'enum', 'VnfPackage',
@@ -280,7 +297,7 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
         'vnfProvider': ('vnfd.vnf_provider', 'string', 'VnfPackageVnfd'),
         'vnfProductName': ('vnfd.vnf_product_name', 'string',
                            'VnfPackageVnfd'),
-        'vnfdId': ('vnfd.vnfd_id', 'string', 'VnfPackageVnfd'),
+        'vnfdId': ('vnfd.vnfd_id', 'uuid', 'VnfPackageVnfd'),
         'vnfSoftwareVersion': ('vnfd.vnf_software_version', 'string',
                                'VnfPackageVnfd'),
         'vnfdVersion': ('vnfd.vnfd_version', 'string', 'VnfPackageVnfd'),
@@ -294,6 +311,7 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
     }
 
     ALL_ATTRIBUTES.update(vnf_software_image.VnfSoftwareImage.ALL_ATTRIBUTES)
+    ALL_ATTRIBUTES.update(vnf_artifact.VnfPackageArtifactInfo.ALL_ATTRIBUTES)
 
     FLATTEN_ATTRIBUTES = utils.flatten_dict(ALL_ATTRIBUTES.copy())
 
@@ -305,6 +323,8 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
     COMPLEX_ATTRIBUTES = ["checksum", "userDefinedData"]
     COMPLEX_ATTRIBUTES.extend(
         vnf_software_image.VnfSoftwareImage.COMPLEX_ATTRIBUTES)
+    COMPLEX_ATTRIBUTES.extend(vnf_artifact.VnfPackageArtifactInfo.
+                              COMPLEX_ATTRIBUTES)
 
     # Version 1.1: Added 'size' to persist size of VnfPackage.
     VERSION = '1.1'
@@ -325,6 +345,8 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
             'VnfDeploymentFlavoursList', nullable=True),
         'vnfd': fields.ObjectField('VnfPackageVnfd', nullable=True),
         'size': fields.IntegerField(nullable=False, default=0),
+        'vnf_artifacts': fields.ObjectField('VnfPackageArtifactInfoList',
+                                            nullable=True)
     }
 
     def __init__(self, context=None, **kwargs):
@@ -375,6 +397,10 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
         if 'vnfd' in expected_attrs:
             vnf_package._load_vnfd(db_vnf_package.get('vnfd'))
 
+        if 'vnf_artifacts' in expected_attrs:
+            vnf_package._load_vnf_artifacts(
+                db_vnf_package.get('vnf_artifacts'))
+
     def _load_vnf_deployment_flavours(self, db_flavours=_NO_DATA_SENTINEL):
         if db_flavours is _NO_DATA_SENTINEL:
             vnf_package = self.get_by_id(
@@ -412,6 +438,25 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
                 self._context, db_vnfd)
             self.obj_reset_changes(['vnfd'])
 
+    def _load_vnf_artifacts(self, db_artifact=_NO_DATA_SENTINEL):
+        if db_artifact is _NO_DATA_SENTINEL:
+            vnf_package = self.get_by_id(
+                self._context, self.id,
+                expected_attrs=['vnf_artifacts'])
+            if 'vnf_artifacts' in vnf_package:
+                self.vnf_artifacts = vnf_package.vnf_artifacts
+                self.vnf_artifacts.obj_reset_changes(recursive=True)
+                self.obj_reset_changes(['vnf_artifacts'])
+            else:
+                self.vnf_artifacts = objects.\
+                    VnfPackageArtifactInfoList(objects=[])
+        elif db_artifact:
+            self.vnf_artifacts = base.obj_make_list(
+                self._context, objects.VnfPackageArtifactInfoList(
+                    self._context), objects.VnfPackageArtifactInfo,
+                db_artifact)
+            self.obj_reset_changes(['vnf_artifacts'])
+
     def _load_generic(self, attrname):
         vnf_package = self.__class__.get_by_id(self._context,
                                                id=self.id,
@@ -449,6 +494,8 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
             self._load_vnf_deployment_flavours()
         elif attrname == 'vnfd':
             self._load_vnfd()
+        elif attrname == 'vnf_artifacts':
+            self._load_vnf_artifacts()
         elif attrname in self.fields and attrname != 'id':
             self._load_generic(attrname)
         else:
@@ -462,9 +509,6 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
 
     @base.remotable
     def create(self):
-        if self.obj_attr_is_set('id'):
-            raise exceptions.ObjectActionError(action='create',
-                                               reason=_('already created'))
         updates = self.obj_get_changes()
 
         if 'id' not in updates:
@@ -475,6 +519,10 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
             if key in updates.keys():
                 updates.pop(key)
 
+        for key in ['vnf_artifacts']:
+            if key in updates.keys():
+                updates.pop(key)
+
         user_data = updates.pop('user_data', None)
         db_vnf_package = _vnf_package_create(self._context, updates,
                                              user_data=user_data)
@@ -513,8 +561,6 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
             # check if vnf package is used by any vnf instances.
             query = context.session.query(
                 func.count(models.VnfInstance.id)).\
-                filter_by(
-                instantiation_state=fields.VnfInstanceState.INSTANTIATED).\
                 filter_by(tenant_id=self.tenant_id).\
                 filter_by(vnfd_id=self.vnfd.vnfd_id).\
                 filter_by(deleted=False)
@@ -607,6 +653,12 @@ class VnfPackage(base.TackerObject, base.TackerPersistentObject,
             if checksum:
                 vnf_package_response.update(checksum)
 
+            artifacts = self.vnf_artifacts.to_dict(
+                include_fields=include_fields)
+            if artifacts:
+                vnf_package_response.update(
+                    {'additionalArtifacts': artifacts})
+
         return vnf_package_response
 
 
