@@ -23,6 +23,8 @@ KEY_SCALE = 'scale_stack_id'
 ACTION_CREATE_VNF = 'create_vnf'
 ACTION_UPDATE_VNF = 'update_vnf'
 ACTION_DELETE_VNF = 'delete_vnf'
+ACTION_INSTANTIATE_VNF = 'instantiate_vnf'
+ACTION_TERMINATE_VNF = 'terminate_vnf'
 ACTION_HEAL_VNF = 'heal_vnf'
 ACTION_SCALE_IN_VNF = 'scale_in_vnf'
 ACTION_SCALE_OUT_VNF = 'scale_out_vnf'
