@@ -19,6 +19,7 @@ Utility methods for working with WSGI servers
 import functools
 
 import errno
+import http.client
 import os
 import socket
 import ssl
@@ -33,19 +34,16 @@ import tacker.conf
 import oslo_i18n as i18n
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
-from oslo_service import service as common_service
 from oslo_service import systemd
 from oslo_utils import encodeutils
 from oslo_utils import excutils
 import routes.middleware
-import six
 import webob.dec
 import webob.exc
 
 from tacker._i18n import _
 from tacker.common import exceptions as exception
 from tacker import context
-from tacker.db import api
 
 
 socket_opts = [
@@ -150,35 +148,6 @@ def expected_errors(errors):
     return decorator
 
 
-class WorkerService(common_service.ServiceBase):
-    """Wraps a worker to be handled by ProcessLauncher."""
-
-    def __init__(self, service, application):
-        self._service = service
-        self._application = application
-        self._server = None
-
-    def start(self):
-        # We may have just forked from parent process.  A quick disposal of the
-        # existing sql connections avoids producing 500 errors later when they
-        # are discovered to be broken.
-        api.get_engine().pool.dispose()
-        self._server = self._service.pool.spawn(self._service._run,
-                                                self._application,
-                                                self._service._socket)
-
-    def wait(self):
-        self._service.pool.waitall()
-
-    def stop(self):
-        if isinstance(self._server, eventlet.greenthread.GreenThread):
-            self._server.kill()
-            self._server = None
-
-    def reset(self):
-        pass
-
-
 class Server(object):
     """Server class to manage multiple WSGI sockets and applications."""
 
@@ -187,7 +156,6 @@ class Server(object):
         eventlet.wsgi.MAX_HEADER_LINE = CONF.max_header_line
         self.pool = eventlet.GreenPool(threads)
         self.name = name
-        self._launcher = None
         self._server = None
 
     def _get_socket(self, host, port, backlog):
@@ -269,7 +237,7 @@ class Server(object):
 
         return sock
 
-    def start(self, application, port, host='0.0.0.0', workers=0):
+    def start(self, application, port, host='0.0.0.0'):
         """Run a WSGI server with the given application."""
         self._host = host
         self._port = port
@@ -278,18 +246,10 @@ class Server(object):
         self._socket = self._get_socket(self._host,
                                         self._port,
                                         backlog=backlog)
-        if workers < 1:
-            # For the case where only one process is required.
-            self._server = self.pool.spawn(self._run, application,
-                                           self._socket)
-            systemd.notify_once()
-        else:
-            # Minimize the cost of checking for child exit by extending the
-            # wait interval past the default of 0.01s.
-            self._launcher = common_service.ProcessLauncher(
-                CONF, wait_interval=1.0, restart_method='mutate')
-            self._server = WorkerService(self, application)
-            self._launcher.launch_service(self._server, workers=workers)
+        # For the case where only one process is required.
+        self._server = self.pool.spawn(self._run, application,
+                                       self._socket)
+        systemd.notify_once()
 
     @property
     def host(self):
@@ -300,19 +260,12 @@ class Server(object):
         return self._socket.getsockname()[1] if self._socket else self._port
 
     def stop(self):
-        if self._launcher:
-            # The process launcher does not support stop or kill.
-            self._launcher.running = False
-        else:
-            self._server.kill()
+        self._server.kill()
 
     def wait(self):
         """Wait until all servers have completed running."""
         try:
-            if self._launcher:
-                self._launcher.wait()
-            else:
-                self.pool.waitall()
+            self.pool.waitall()
         except KeyboardInterrupt:
             pass
 
@@ -404,11 +357,11 @@ class Request(webob.Request):
         type_from_header = self.get_content_type()
         if type_from_header:
             return type_from_header
-        ctypes = ['application/json', 'text/plain', 'application/zip']
+        offers = ['application/json', 'text/plain', 'application/zip']
 
         # Finally search in Accept-* headers
-        bm = self.accept.best_match(ctypes)
-        return bm or 'application/json'
+        ctypes = self.accept.acceptable_offers(offers)
+        return ctypes[0][0] if ctypes else 'application/json'
 
     def get_content_type(self):
         allowed_types = ("application/json", "application/zip")
@@ -466,7 +419,7 @@ class JSONDictSerializer(DictSerializer):
 
     def default(self, data):
         def sanitizer(obj):
-            return six.text_type(obj)
+            return str(obj)
         return encode_body(jsonutils.dump_as_bytes(data, default=sanitizer))
 
 
@@ -955,26 +908,17 @@ class ResponseObject(object):
         response = webob.Response(body=body)
         response.status_int = self.code
         for hdr, val in self._headers.items():
-            if six.PY2:
-                # In Py2.X Headers must be a UTF-8 encode str.
-                response.headers[hdr] = encodeutils.safe_encode(val)
-            else:
-                # In Py3.X Headers must be a str that was first safely
-                # encoded to UTF-8 (to catch any bad encodings) and then
-                # decoded back to a native str.
-                response.headers[hdr] = encodeutils.safe_decode(
-                    encodeutils.safe_encode(val))
+            # Headers must be a str that was first safely
+            # encoded to UTF-8 (to catch any bad encodings) and then
+            # decoded back to a native str.
+            response.headers[hdr] = encodeutils.safe_decode(
+                encodeutils.safe_encode(val))
         # Deal with content_type
-        if not isinstance(content_type, six.text_type):
-            content_type = six.text_type(content_type)
-        if six.PY2:
-            # In Py2.X Headers must be a UTF-8 encode str.
-            response.headers['Content-Type'] = encodeutils.safe_encode(
-                content_type)
-        else:
-            # In Py3.X Headers must be a str.
-            response.headers['Content-Type'] = encodeutils.safe_decode(
-                encodeutils.safe_encode(content_type))
+        if not isinstance(content_type, str):
+            content_type = str(content_type)
+
+        response.headers['Content-Type'] = encodeutils.safe_decode(
+            encodeutils.safe_encode(content_type))
         return response
 
     @property
@@ -1095,19 +1039,6 @@ def _default_body_function(wrapped_exc):
 class Fault(webob.exc.HTTPException):
     """Wrap webob.exc.HTTPException to provide API friendly response."""
 
-    _fault_names = {
-        400: "badRequest",
-        401: "unauthorized",
-        403: "forbidden",
-        404: "itemNotFound",
-        405: "badMethod",
-        409: "conflictingRequest",
-        413: "overLimit",
-        415: "badMediaType",
-        429: "overLimit",
-        501: "notImplemented",
-        503: "serviceUnavailable"}
-
     def __init__(self, exception):
         """Create a Fault for the given webob.exc.exception."""
         self.wrapped_exc = exception
@@ -1121,26 +1052,28 @@ class Fault(webob.exc.HTTPException):
         user_locale = req.best_match_language()
         # Replace the body with fault details.
         code = self.wrapped_exc.status_int
-        fault_name = self._fault_names.get(code, "tackerFault")
+        fault_name = http.client.responses[code]
         explanation = self.wrapped_exc.explanation
         LOG.debug("Returning %(code)s to user: %(explanation)s",
                   {'code': code, 'explanation': explanation})
 
         explanation = i18n.translate(explanation, user_locale)
-        fault_data = {
-            fault_name: {
-                'code': code,
-                'message': explanation}}
+        fault_data = {}
+        if fault_name is not None:
+            fault_data['title'] = fault_name
+        fault_data['status'] = code
+        fault_data['detail'] = explanation
+
         if code == 413 or code == 429:
             retry = self.wrapped_exc.headers.get('Retry-After', None)
             if retry:
-                fault_data[fault_name]['retryAfter'] = retry
+                fault_data['retryAfter'] = retry
 
-        self.wrapped_exc.content_type = 'application/json'
+        self.wrapped_exc.content_type = 'application/problem+json'
         self.wrapped_exc.charset = 'UTF-8'
 
         body = JSONDictSerializer().serialize(fault_data)
-        if isinstance(body, six.text_type):
+        if isinstance(body, str):
             body = body.encode('utf-8')
         self.wrapped_exc.body = body
 
